% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RemoteSensing.R
\docType{class}
\name{RemoteSensing}
\alias{RemoteSensing}
\title{RemoteSensing class}
\format{
RemoteSensing \code{\link{R6Class}} object.

Instance objects of R6Class are created by 'new':

\code{remotesensing <- RemoteSensing$new(url, userpwd=NULL)}

Methods of instance objects are called by '$':

\code{result <- remotesensing$method(parameters)}
}
\usage{
remotesensing <- RemoteSensing$new(url, userpwd=NULL)

remotesensing$rasterdbs
remotesensing$rasterdb(name)
remotesensing$create_rasterdb(name)

remotesensing$pointclouds
remotesensing$pointcloud(name)

remotesensing$pointdbs
remotesensing$pointdb(name)

remotesensing$roi_groups
remotesensing$roi_group(name)
remotesensing$roi(group_name, roi_name)

remotesensing$poi_groups
remotesensing$poi_group(name)
remotesensing$poi(group_name, poi_name)

remotesensing$web()
}
\description{
Remote sensing database manages (hyperspectral) \strong{rasters} and (LiDAR) \strong{point-clouds}
as well as auxiliary ROIs (regions of interest as named polygons) and POIs (points of interest as named points).
}
\details{
Objects of RemoteSensing class encapsulate connections to one remote sensing database.
}
\section{Methods}{


\describe{

\item{RemoteSensing$new(url, userpwd=NULL)}{Open Remote Sensing Database.

url: url of server. (local e.g. "http://localhost:8081" or remote e.g. "http://example.com:8081")

userpwd: optional authentication (format: "user:password")

returns: RemoteSensing object}

\item{$rasterdbs}{get names of RasterDBs contained in Remote Sensing Database.}

\item{$rasterdb(name)}{get RasterDB by name.}

\item{$create_rasterdb(name)}{creates new empty RasterDB.

returns: success message}

\item{$pointclouds}{get names of PointClouds contained in Remote Sensing Database.}

\item{$pointcloud(name)}{get PointCloud by name.}

\item{$pointdbs}{get names of PointDBs contained in Remote Sensing Database.}

\item{$pointdb(name)}{get PointDB by name.}

\item{$roi_groups}{get names of ROI-groups.

returns: data.frame of name and description}

\item{$roi_group(name)}{get ROI-group by name.

returns: data.frame of ROI-name and polygon}

\item{$roi(group_name, roi_name)}{get one ROI.

returns: polygon}

\item{$poi_groups}{get names of POI-groups.

returns: data.frame of name and description}

\item{$poi_group(name)}{get POI-group by name.

returns: data.frame of POI-name and position}

\item{$poi(group_name, poi_name)}{get one POI.

returns: position}

\item{$web()}{Open web interface in browser.}
}
}

\section{ROI}{

Region of interest (ROI) is a named polygon.

Matrix of one row per polygon vertex and two columns with x- and y-coordinates describe the polygon.

First and last vertex need to be same (a closed polygon).

example:
\preformatted{
# get one ROI
roi <- remotesensing$roi(group_name="kili_A", roi_name="cof3_A")

# get name of ROI
roi$name

# get matrix of points
roi$polygon[[1]]
}
}

\section{POI}{

Point of interest (POI) is a named point.

example:
\preformatted{
# get one POI
poi <- remotesensing$poi(group_name="kili", poi_name="cof3")

# get name of ROI
poi$name

# get x-coodinate of POI
poi$x

# get y-coodinate of POI
poi$y
}
}

\examples{
# open remote sensing database
library(rPointDB)
# remotesensing <- RemoteSensing$new("http://localhost:8081", "user:password") # local
remotesensing <- RemoteSensing$new("http://example.com:8081", "user:password") # remote server

# get names of RasterDBs
remotesensing$rasterdbs

# get one rasterdb
rasterdb <- remotesensing$rasterdb("kili_campaign1")

# get names of PointDBs
remotesensing$pointdbs

# get one pointdb
pointdb <- remotesensing$pointdb("kili")

# get names of ROI groups
remotesensing$roi_groups

# get one ROI group
rois <- remotesensing$roi_group("kili_A")

# get one ROI
roi <- remotesensing$roi(group_name="kili_A", roi_name="cof3_A")

# get names of POI groups
remotesensing$poi_groups

# get one POI group
pois <- remotesensing$poi_group("kili")

# get one POI
poi <- remotesensing$poi(group_name="kili", poi_name="cof3")

# create extent around POI of 10 meter edge length
ext <- extent_diameter(poi$x, poi$y, 10)

# get RasterStack of all bands at ext
r <- rasterdb$raster(ext)

# get data.frame of LiDAR points at ext
df1 <- pointdb$query(ext)

#get data.frame of LiDAR points at polygon ROI
df2 <- pointdb$query_polygon(roi$polygon[[1]])

# open web interface in browser
remotesensing$web()

}
\seealso{
\link{RasterDB} \link{PointCloud} \link{PointDB}
}
\author{
woellauer
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-new}{\code{RemoteSensing$new()}}
\item \href{#method-lidar}{\code{RemoteSensing$lidar()}}
\item \href{#method-pointdb}{\code{RemoteSensing$pointdb()}}
\item \href{#method-rasterdb}{\code{RemoteSensing$rasterdb()}}
\item \href{#method-lidar_layers}{\code{RemoteSensing$lidar_layers()}}
\item \href{#method-web}{\code{RemoteSensing$web()}}
\item \href{#method-poi_group}{\code{RemoteSensing$poi_group()}}
\item \href{#method-poi}{\code{RemoteSensing$poi()}}
\item \href{#method-roi_group}{\code{RemoteSensing$roi_group()}}
\item \href{#method-roi}{\code{RemoteSensing$roi()}}
\item \href{#method-create_rasterdb}{\code{RemoteSensing$create_rasterdb()}}
\item \href{#method-pointcloud}{\code{RemoteSensing$pointcloud()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-new"></a>}}
\if{latex}{\out{\hypertarget{method-new}{}}}
\subsection{Method \code{new()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$new(url, userpwd = NULL, ssl_verifypeer = TRUE)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-lidar"></a>}}
\if{latex}{\out{\hypertarget{method-lidar}{}}}
\subsection{Method \code{lidar()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$lidar(layer)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-pointdb"></a>}}
\if{latex}{\out{\hypertarget{method-pointdb}{}}}
\subsection{Method \code{pointdb()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$pointdb(name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-rasterdb"></a>}}
\if{latex}{\out{\hypertarget{method-rasterdb}{}}}
\subsection{Method \code{rasterdb()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$rasterdb(name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-lidar_layers"></a>}}
\if{latex}{\out{\hypertarget{method-lidar_layers}{}}}
\subsection{Method \code{lidar_layers()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$lidar_layers()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-web"></a>}}
\if{latex}{\out{\hypertarget{method-web}{}}}
\subsection{Method \code{web()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$web()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-poi_group"></a>}}
\if{latex}{\out{\hypertarget{method-poi_group}{}}}
\subsection{Method \code{poi_group()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$poi_group(group_name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-poi"></a>}}
\if{latex}{\out{\hypertarget{method-poi}{}}}
\subsection{Method \code{poi()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$poi(group_name, poi_name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-roi_group"></a>}}
\if{latex}{\out{\hypertarget{method-roi_group}{}}}
\subsection{Method \code{roi_group()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$roi_group(group_name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-roi"></a>}}
\if{latex}{\out{\hypertarget{method-roi}{}}}
\subsection{Method \code{roi()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$roi(group_name, roi_name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-create_rasterdb"></a>}}
\if{latex}{\out{\hypertarget{method-create_rasterdb}{}}}
\subsection{Method \code{create_rasterdb()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$create_rasterdb(name)}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-pointcloud"></a>}}
\if{latex}{\out{\hypertarget{method-pointcloud}{}}}
\subsection{Method \code{pointcloud()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RemoteSensing$pointcloud(name)}\if{html}{\out{</div>}}
}

}
}
