package server.api.poi_groups;

import java.io.IOException;

import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;


import org.tinylog.Logger;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Response;
import org.eclipse.jetty.server.UserIdentity;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.json.JSONWriter;

import broker.Broker;
import broker.group.PoiGroup;
import util.JsonUtil;
import util.Web;

public class APIHandler_poi_groups extends AbstractHandler {

	private final Broker broker;
	private final APIHandler_poi_group apihandler_poi_group;	

	public APIHandler_poi_groups(Broker broker) {
		this.broker = broker;
		apihandler_poi_group = new APIHandler_poi_group(broker);
	}

	@Override
	public void handle(String target, Request request, HttpServletRequest r, HttpServletResponse response) throws IOException, ServletException {
		request.setHandled(true);
		UserIdentity userIdentity = Web.getUserIdentity(request);
		try {
			if(target.equals("/")) {
				handleList(request, response, userIdentity);
			} else {
				int i = target.indexOf('/', 1);
				if(i == 1) {
					throw new RuntimeException("no name in poi_groups: "+target);
				}			
				String name = i < 0 ? target.substring(1) : target.substring(1, i);
				String next = i < 0 ? "/" : target.substring(i);
				apihandler_poi_group.handle(name, next, request, (Response)response, userIdentity);
			}
		} catch(Exception e) {
			e.printStackTrace();
			Logger.error(e);
			response.setStatus(HttpServletResponse.SC_NOT_FOUND);
			response.setContentType(Web.MIME_TEXT);
			response.getWriter().println(e);
		}
	}

	private void handleList(Request request, HttpServletResponse response, UserIdentity userIdentity) throws IOException {
		response.setStatus(HttpServletResponse.SC_OK);
		response.setContentType(Web.MIME_JSON);
		JSONWriter json = new JSONWriter(response.getWriter());
		json.object();
		json.key("poi_groups");
		json.array();
		for (PoiGroup group : broker.getPoiGroups()) {
			if(group.acl.isAllowed(userIdentity)) {
				json.object();
				json.key("name");
				json.value(group.name);
				json.key("title");
				json.value(group.informal.title);
				JsonUtil.writeOptList(json, "tags", group.informal.tags);
				json.endObject();
			}
		}
		json.endArray();
		json.endObject();
	}
}